@@ -35,11 +35,7 @@
 #include "gtlscertificate-openssl.h"
 #include <glib/gi18n-lib.h>
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined (LIBRESSL_VERSION_NUMBER)
-#  define DEFAULT_CIPHER_LIST "HIGH:!DSS:!aNULL@STRENGTH"
-#else
-#  define DEFAULT_CIPHER_LIST "PROFILE=SYSTEM"
-#endif
+#define DEFAULT_CIPHER_LIST "HIGH:!DSS:!aNULL@STRENGTH"
 
 typedef struct _GTlsClientConnectionOpensslPrivate
 {
@@ -284,17 +280,6 @@ g_tls_client_connection_openssl_get_ssl (GTlsConnectionOpenssl *connection)
   return priv->ssl;
 }
 
-static SSL_CTX *
-g_tls_client_connection_openssl_get_ssl_ctx (GTlsConnectionOpenssl *connection)
-{
-  GTlsClientConnectionOpenssl *client = G_TLS_CLIENT_CONNECTION_OPENSSL (connection);
-  GTlsClientConnectionOpensslPrivate *priv;
-
-  priv = g_tls_client_connection_openssl_get_instance_private (client);
-
-  return priv->ssl_ctx;
-}
-
 static void
 g_tls_client_connection_openssl_class_init (GTlsClientConnectionOpensslClass *klass)
 {
@@ -311,7 +296,6 @@ g_tls_client_connection_openssl_class_init (GTlsClientConnectionOpensslClass *kl
   base_class->complete_handshake = g_tls_client_connection_openssl_complete_handshake;
 
   connection_class->get_ssl = g_tls_client_connection_openssl_get_ssl;
-  connection_class->get_ssl_ctx = g_tls_client_connection_openssl_get_ssl_ctx;
 
   g_object_class_override_property (gobject_class, PROP_VALIDATION_FLAGS, "validation-flags");
   g_object_class_override_property (gobject_class, PROP_SERVER_IDENTITY, "server-identity");
@@ -414,8 +398,9 @@ generate_session_id (SSL           *ssl,
   return 1;
 }
 
-static void
-set_cipher_list (GTlsClientConnectionOpenssl *client)
+static gboolean
+set_cipher_list (GTlsClientConnectionOpenssl  *client,
+                 GError                      **error)
 {
   GTlsClientConnectionOpensslPrivate *priv;
   const gchar *cipher_list;
@@ -426,7 +411,15 @@ set_cipher_list (GTlsClientConnectionOpenssl *client)
   if (cipher_list == NULL)
     cipher_list = DEFAULT_CIPHER_LIST;
 
-  SSL_CTX_set_cipher_list (priv->ssl_ctx, cipher_list);
+  if (!SSL_CTX_set_cipher_list (priv->ssl_ctx, cipher_list))
+    {
+      g_set_error (error, G_TLS_ERROR, G_TLS_ERROR_MISC,
+                   _("Could not create TLS context: %s"),
+                   ERR_error_string (ERR_get_error (), NULL));
+      return FALSE;
+    }
+
+  return TRUE;
 }
 
 #ifdef SSL_CTX_set1_sigalgs_list
@@ -492,6 +485,9 @@ g_tls_client_connection_openssl_initable_init (GInitable       *initable,
       return FALSE;
     }
 
+  if (!set_cipher_list (client, error))
+    return FALSE;
+
   /* Only TLS 1.2 or higher */
   options = SSL_OP_NO_TICKET |
             SSL_OP_NO_COMPRESSION |
@@ -519,13 +515,12 @@ g_tls_client_connection_openssl_initable_init (GInitable       *initable,
     }
 #endif
 
-  SSL_CTX_set_generate_session_id (priv->ssl_ctx, generate_session_id);
+  SSL_CTX_set_generate_session_id (priv->ssl_ctx, (GEN_SESSION_CB)generate_session_id);
+
   SSL_CTX_add_session (priv->ssl_ctx, priv->session);
 
   SSL_CTX_set_client_cert_cb (priv->ssl_ctx, retrieve_certificate);
 
-  set_cipher_list (client);
-
 #ifdef SSL_CTX_set1_sigalgs_list
   set_signature_algorithm_list (client);
 #endif
